package net.pricefx.formulaengine.scripting;

import java.io.Closeable;

/**
 *
 * @see <a href="http://www.rforge.net/org/doc/org/rosuda/REngine/REXP.html">REXP</a>
 */
public interface RContext extends Closeable {

	/**
	 * Assigns a value to a variable in R.  The following types can be assigned
	 * <p><ul>
	 *     <li>String</li>
	 *     <li>String[]</li>
	 *     <li>int[]</li>
	 *     <li>double[]</li>
	 *     <li><a href="https://www.rforge.net/org/docs/org/rosuda/REngine/REXP.html">REXP</a></li>
	 *     <li>{@link net.pricefx.formulaengine.scripting.Matrix2D}</li>
	 * </ul></p>
	 * A value of type REXP would be one obtained from R with an <code>eval</code> or <code>get</code> call.
	 * A Matrix2D object is the result of a PA query: {@link net.pricefx.formulaengine.DatamartContext#executeQuery}
	 *
	 * @param var Variable to assign to
	 * @param value	Value(s) to assign
	 * @see <a href="http://www.rforge.net/org/doc/org/rosuda/REngine/REXP.html">REXP</a>
	 */
	void assign(String var, Object value);

	/**
	 * Parses and evaluates an R expression and returns the result.
	 * An expression can be as short as or as long as needed (i.e. a whole script).
	 *
	 * @param expr  The expression, as string, to evaluate
	 * @return	The resulting expression or null in case of an error (see <a href="http://www.rforge.net/org/doc/org/rosuda/REngine/REXP.html">REXP</a>)
	 * @see <a href="http://www.rforge.net/org/doc/org/rosuda/REngine/REXP.html">REXP</a>
	 */
	Object eval(String expr);

	/**
	 * Retreives a value from R
	 * @param var	The variable to retreive the value from
	 * @return	The resulting expression (see <a href="http://www.rforge.net/org/doc/org/rosuda/REngine/REXP.html">REXP</a>)
	 * @see <a href="http://www.rforge.net/org/doc/org/rosuda/REngine/REXP.html">REXP</a>
	 */
	Object get(String var);

	/**
	 * Often in R, results have a 'names' attribute, for example the parameter names in an lm() regression
	 * @param var	The variable to retreive the names from
	 * @return	The names array, or null if there is no such attribute
	 */
	String[] names(String var);

	/**
	 * When evaluating an expression in R some output may be printed as a side effect, such as by lm().
	 * While not structured data, it may be useful to capture this output to show as a calculation result in the UI, or merely for logging purposes.
	 * @param var	The variable to print, i.e. where that expression was assigned to
	 * @return	The printed output as a string
	 */
	String print(String var);

	/**
	 * Future functionality - Not yet implemented!
	 * @param expr
	 * @param fileName
	 */
	void plot(String expr, String fileName);

}