package net.pricefx.formulaengine.scripting.inputbuilder;

import java.util.HashMap;
import java.util.Map;

import net.pricefx.common.api.InputType;
import net.pricefx.common.api.PriceFxInterface;

public abstract class AbstractProductAndCustomerInputBuilder<T extends AbstractProductAndCustomerInputBuilder, VT> extends AbstractInputBuilder<T, VT> {

    protected String attributeName;
    protected String filterFormulaName;
    protected String filterFormulaParam;

    protected AbstractProductAndCustomerInputBuilder(InputBuilderContext builderContext, InputType type, String paramName) {
        super(builderContext, type, paramName);
    }

    /**
     * Set attribute name.
     * <p>Example:
     * <blockquote><pre>
     * def customer = api.inputBuilderFactory()
     *                  .createCustomerEntry("Customer")
     *                  .setAttributeName("Test")
     * </pre></blockquote>
     *
     * @param attributeName Name of the attribute
     * @return self
     */
    public T setAttributeName(String attributeName) {
        this.attributeName = attributeName;
        return (T) this;
    }

    /**
     * Set filter formula parameter.
     * <p>Example:
     * <blockquote><pre>
     * def customer = api.inputBuilderFactory()
     *                  .createCustomerEntry("Customer")
     *                  .setLabel("Customer")
     *                  .setRequired(false)
     *                  .setNoRefresh(true)
     *                  .setFilterFormulaName("CustomerPickerFilter")
     *                  .setFilterFormulaParam(api.jsonEncode([CountryUnit: 'CZ', CustomerGroupNo: null]))
     *                  .buildContextParameter()
     * </pre></blockquote>
     *
     * @param filterFormulaName Filter formula name
     * @return self
     */
    public T setFilterFormulaName(String filterFormulaName) {
        this.filterFormulaName = filterFormulaName;
        Map<String, Object> filterFormula = getOrCreateFilterFormula();
        filterFormula.put(PriceFxInterface.FIELD_CONTEXTPARAM_CONFIG_FILTERFORMULA_NAME, filterFormulaName);
        cp.setConfigParameter(PriceFxInterface.PARAM_CM_FILTERFORMULA, filterFormula);
        return (T) this;
    }

    /**
     * Set filter formula parameter.
     * <p>Example:
     * <blockquote><pre>
     * def customer = api.inputBuilderFactory()
     *                  .createCustomerEntry("Customer")
     *                  .setLabel("Customer")
     *                  .setRequired(false)
     *                  .setNoRefresh(true)
     *                  .setFilterFormulaName("CustomerPickerFilter")
     *                  .setFilterFormulaParam(api.jsonEncode([CountryUnit: 'CZ', CustomerGroupNo: null]))
     *                  .buildContextParameter()
     * </pre></blockquote>
     *
     * @param filterFormulaParam Filter formula parameter
     * @return self
     */
    public T setFilterFormulaParam(String filterFormulaParam) {
        this.filterFormulaParam = filterFormulaParam;
        Map<String, Object> filterFormula = getOrCreateFilterFormula();
        filterFormula.put(PriceFxInterface.FIELD_CONTEXTPARAM_CONFIG_FILTERFORMULA_PARAM, filterFormulaParam);
        cp.setConfigParameter(PriceFxInterface.PARAM_CM_FILTERFORMULA, filterFormula);
        return (T) this;
    }

    protected Map<String, Object> getOrCreateFilterFormula() {
        return (Map<String, Object>) cp.getParameterConfig().getOrDefault(PriceFxInterface.PARAM_CM_FILTERFORMULA, new HashMap<>());
    }

}
