package net.pricefx.formulaengine.scripting.inputbuilder;

import net.pricefx.common.api.InputType;
import net.pricefx.common.api.PriceFxInterface;

import java.util.Date;
import java.util.List;
import java.util.Optional;

/**
 * Builder class for basic input types.
 */
public class SimpleInputBuilder<VT> extends AbstractInputBuilder<SimpleInputBuilder, VT> {

    private final String functionName;

    protected SimpleInputBuilder(InputBuilderContext builderContext, String paramName, InputType inputType, String functionName) {
        super(builderContext, inputType, paramName);
        this.functionName = functionName;
    }

    /**
     * Sets the data type of the input field.<br>
     * Example:
     * <blockquote><pre>
     * def dateFormat = "yyyy-MM-dd"
     * def maxDate = api.parseDate(dateFormat, "2021-10-17")
     * def dateEntry = api.inputBuilderFactory()
     *                      .createDateUserEntry("date")
     *                      .setLabel("Date")
     *                      .setPlaceholderText("date to 2021-10-17")
     *                      .setDataType("DATE")
     *                      .setFormatType(dateFormat)
     *                      .setTo(maxDate)
     *                      .buildContextParameter()
     * </pre></blockquote>
     *
     * @param dataType The type of the data.
     * @return self
     */
    public SimpleInputBuilder setDataType(String dataType) {
        if (dataType != null)
            cp.setConfigParameter(PriceFxInterface.FIELD_CONTEXTPARAM_CONFIG_DATATYPE, dataType);
        return this;
    }

    /**
     * Sets the format type of the input field.<br>
     * Example:
     * <blockquote><pre>
     * def userEntry = api.inputBuilderFactory()
     *                .createUserEntry("Percentage")
     *                .setLabel("Percentage")
     *                .setPlaceholderText("Value between 35% and 60%")
     *                .setFormatType("PERCENT")
     *                .setFrom(0.35)
     *                .setTo(0.60)
     * </pre></blockquote>
     *
     * @param formatType The maximum value.
     * @return self
     */
    public SimpleInputBuilder setFormatType(String formatType) {
        cp.setConfigParameter(PriceFxInterface.FIELD_CONTEXTPARAM_CONFIG_FORMATTYPE, formatType);
        return this;
    }

    /**
     * Sets the input type of the field.<br>
     * Example:
     *
     * @param inputType Type fo the input, ex. "range"
     * @return self
     */
    public SimpleInputBuilder setInputType(String inputType) {
        cp.setConfigParameter(PriceFxInterface.FIELD_CONTEXTPARAM_CONFIG_INPUTTYPE, inputType);
        return this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.
     *
     * @param doRound The rounding mechanism turn on (true) or off (false)
     * @return self
     */
    public SimpleInputBuilder setRounding(boolean doRound) {
        cp.setConfigParameter("rounding", doRound);
        return this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.
     *
     * @param mode The decimal places of an element to display in the UI
     * @return self
     */
    public SimpleInputBuilder setDecimalPlaces(String mode) {
        cp.setConfigParameter("decimalPlaces", mode);
        return this;
    }

    /**
     * Sets the maximum allowed value for the entry.
     * The property set with this method is 'to'.
     * Please notice that this method only accepts integer values, and no overloaded version of this method that accepts
     * decimals exist. To use decimals, please use the replacement, {@link #setTo(Float) setTo} .
     *
     * @param max maximum value
     * @return self
     * @deprecated as of 7.7.0, use instead {@link #setTo(Float) setTo}
     */
    @Deprecated(since = "7.7.0", forRemoval = true)
    public SimpleInputBuilder setMax(Integer max) {
        return setTo(Optional.ofNullable(max).map(Integer::floatValue).orElse(null));
    }

    /**
     * Sets the maximum allowed value for the input field. When set, a user will not be able to select a higher value than specified by this method.<br>
     * Applicable to the following input types: INTEGERUSERENTRY, USERENTRY<br>
     * Example:
     * <blockquote><pre>
     * def userEntry = api.inputBuilderFactory()
     *                .createUserEntry("Percentage")
     *                .setLabel("Percentage")
     *                .setPlaceholderText("Value between 10% and 20%")
     *                .setFormatType("PERCENT")
     *                .setFrom(0.1)
     *                .setTo(0.2)
     * </pre></blockquote>
     *
     * @param to The maximum value.
     * @return self
     * @since 8.0 - Godfather
     */
    public SimpleInputBuilder setTo(Float to) {
        cp.setConfigParameter("to", to);
        return this;
    }

    /**
     * Sets the maximum allowed date value for the entry.
     * The property set with this method is 'to'
     *
     * @param max maximum date value
     * @return self
     * @deprecated as of 7.7.0, use instead {@link #setTo(Date) setTo}
     */
    @Deprecated(since = "7.7.0", forRemoval = true)
    public SimpleInputBuilder setMax(Date max) {
        return setTo(max);
    }

    /**
     * Sets the maximum allowed date value for the input field. When set, a user will not be able to select a higher date than specified by this method.<br>
     * Use together with {@link #setFrom(Date)} to specify the date range.<p>
     * Applicable to the following input types: DATETIMEUSERENTRY (accepts date limits only), DATEUSERENTRY<br>
     * Example:
     * <blockquote><pre>
     * def dateFormat = "yyyy-MM-dd"
     * def maxDate = api.parseDate(dateFormat, "2021-10-17")
     * def dateEntry = api.inputBuilderFactory()
     *      .createDateUserEntry("date")
     *      .setLabel("Date")
     *      .setPlaceholderText("date to 2021-10-17")
     *      .setFormatType(dateFormat)
     *      .setTo(maxDate)
     * </pre></blockquote>
     *
     * @param to The maximum date.
     * @return self
     * @since 8.0 - Godfather
     */
    public SimpleInputBuilder setTo(Date to) {
        cp.setConfigParameter("to", to);
        return this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.
     *
     * @param maxLen The maximum length allowed for the input field.
     * @return self
     */
    public SimpleInputBuilder setMaxLength(Integer maxLen) {
        cp.setConfigParameter("maxLength", maxLen);
        return this;
    }

    /**
     * Sets the minimum allowed value for the entry.
     * The property set with this method is 'from'.
     * Please notice that this method only accepts integer values, and no overloaded version of this method that accepts
     * decimals exist. To use decimals, please use the replacement, {@link #setFrom(Float) setFrom} .
     *
     * @param min minimum value
     * @return self
     * @deprecated as of 7.7.0, use instead {@link #setFrom(Float) setFrom}
     */
    @Deprecated(since = "7.7.0", forRemoval = true)
    public SimpleInputBuilder setMin(Integer min) {
        return setFrom(Optional.ofNullable(min).map(Integer::floatValue).orElse(null));
    }

    /**
     * Sets the minimum allowed value for the input field. When set, a user will not be able to select a lower value than specified by this method.<br>
     * Applicable to the following input types: INTEGERUSERENTRY, USERENTRY<p>
     * Example:
     * <blockquote><pre>
     * def userEntry = api.inputBuilderFactory()
     *                .createUserEntry("Percentage")
     *                .setLabel("Percentage")
     *                .setPlaceholderText("Value between 10% and 20%")
     *                .setFormatType("PERCENT")
     *                .setFrom(0.1)
     *                .setTo(0.2)
     * </pre></blockquote>
     *
     * @param from The minimum value.
     * @return self
     * @since 8.0 - Godfather
     */
    public SimpleInputBuilder setFrom(Float from) {
        cp.setConfigParameter("from", from);
        return this;
    }

    /**
     * Sets the minimum allowed date value for the entry.
     * The property set with this method is 'from'
     *
     * @param min minimum date value
     * @return self
     * @deprecated as of 7.7.0, use instead {@link #setFrom(Date) setFrom}
     */
    @Deprecated(since = "7.7.0", forRemoval = true)
    public SimpleInputBuilder setMin(Date min) {
        return setFrom(min);
    }

    /**
     * Sets the minimum allowed date value for the input field. When set, a user will not be able to select a lower date than specified by this method.<br>
     * Use together with {@link #setTo(Date)} to specify the date range.<p>
     * Applicable to the following input types: DATETIMEUSERENTRY (accepts date limits only), DATEUSERENTRY<p>
     * Example:
     * <blockquote><pre>
     * def dateFormat = "yyyy-MM-dd"
     * def minDate = api.parseDate(dateFormat, "2021-10-02")
     * def dateEntry = api.inputBuilderFactory()
     *      .createDateUserEntry("date")
     *      .setLabel("Date")
     *      .setPlaceholderText("date from 2021-10-02")
     *      .setFormatType(dateFormat)
     *      .setFrom(minDate)
     * </pre></blockquote>
     *
     * @param from The minimum date.
     * @return self
     * @return self
     * @since 8.0 - Godfather
     */
    public SimpleInputBuilder setFrom(Date from) {
        cp.setConfigParameter("from", from);
        return this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.
     *
     * @param minLen The minimum length allowed for the input field.
     * @return self
     */
    public SimpleInputBuilder setMinLength(Integer minLen) {
        cp.setConfigParameter("minLength", minLen);
        return this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.
     *
     * @param list To be defined
     * @return self
     */
    public SimpleInputBuilder setList(List list) {
        cp.setConfigParameter("list", list);
        return this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.
     *
     * @param rte true for the usage of RichTextEditor
     * @return self
     */
    public SimpleInputBuilder setRichTextEditor(boolean rte) {
        cp.setFormattingOption("richTextEditor", rte);
        return this;
    }
}
