package net.pricefx.logic.core.model;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.databind.node.ObjectNode;
import net.pricefx.common.api.IFormulaNature;
import net.pricefx.common.api.PriceFxInterface;
import net.pricefx.pckg.transform.TransformCalculationLogic;
import net.pricefx.pckg.transform.TransformWorkflowFormula;
import net.pricefx.pckg.transform.descriptor.TypeDescriptor;
import net.pricefx.restapiclient.model.BaseMap;

import java.nio.file.Path;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.stream.Collectors;

import static net.pricefx.common.api.FormulaNature.CFO_HEADER;
import static net.pricefx.common.api.FormulaNature.COMPENSATION_HEADER;
import static net.pricefx.common.api.FormulaNature.CO_HEADER;
import static net.pricefx.common.api.FormulaNature.CONTRACT_HEADER;
import static net.pricefx.common.api.FormulaNature.QUOTE_HEADER;
import static net.pricefx.common.api.FormulaNature.REBATE_AGREEMENT_HEADER;
import static net.pricefx.common.api.PriceFxInterface.FIELD_FE_ELEMENTS;
import static net.pricefx.common.api.PriceFxInterface.FIELD_FE_FORMULANATURE;
import static net.pricefx.common.api.PriceFxInterface.FIELD_LABEL;
import static net.pricefx.common.api.PriceFxInterface.FIELD_SIMULATIONSET;
import static net.pricefx.common.api.PriceFxInterface.FIELD_STATUS;
import static net.pricefx.common.api.PriceFxInterface.FIELD_TYPEDID;
import static net.pricefx.common.api.PriceFxInterface.FIELD_UNIQUENAME;
import static net.pricefx.common.api.PriceFxInterface.FIELD_VALIDAFTER;
import static net.pricefx.common.api.PriceFxInterface.FIELD_VERSION;


public abstract class AbstractFormula<E extends AbstractFormulaElement> extends ObjectHolder implements Iterable<AbstractFormulaElement> {

	public static final String LOGIC_NAME = "StudioEvaluator";
	public static final String ELEMENT_DEBUG = "result";

	public AbstractFormula() {
		setListValue(FIELD_FE_ELEMENTS, new ArrayList<>());
	}

	public static <T extends AbstractFormula> T fromObjectNode(ObjectNode node, Class<T> clazz) {
		T logic = fromObject(node, clazz);
		logic.convertElements();
		return logic;
	}

	public static AbstractFormula createLogic(ObjectNode node, TypeDescriptor objectType, Path modulePath) {
		AbstractFormula logic;

		if (objectType == TransformCalculationLogic.DESCRIPTOR) {
			logic = fromObjectNode(node, PricingFormula.class);
		} else if (objectType == TransformWorkflowFormula.DESCRIPTOR) {
			logic = fromObjectNode(node, WorkflowFormula.class);
		} else {
			throw new IllegalArgumentException("Cannot create logic for " + objectType.getName());
		}

		logic.setDirPathBasedOnFsId(modulePath);

		return logic;
	}

	public TypeDescriptor getTypeDescriptor() {
		if (this instanceof PricingFormula) {
			return TransformCalculationLogic.DESCRIPTOR;
		} else if (this instanceof WorkflowFormula) {
			return TransformCalculationLogic.DESCRIPTOR;
		} else {
			throw new IllegalArgumentException("Cannot get TypeDescriptor for " + this.getUniqueName());
		}
	}

	public static AbstractFormula createLogic(TypeDescriptor objectType) {
		if (objectType == TransformCalculationLogic.DESCRIPTOR) {
			return new PricingFormula();
		} else if (objectType == TransformWorkflowFormula.DESCRIPTOR) {
			return new WorkflowFormula();
		} else {
			throw new IllegalArgumentException("Cannot create logic for " + objectType.getName());
		}
	}

	public static AbstractFormula createVirtualLogic(TypeDescriptor objectType) {
		AbstractFormula virtualLogic = createLogic(objectType);
		virtualLogic.setUniqueName(LOGIC_NAME);

		AbstractFormulaElement element = virtualLogic.createElement();
		element.setElementName(ELEMENT_DEBUG);
		element.setCombinationType(ElementCombinationType.FUNCTION.name());
		virtualLogic.addElement(element);

		return virtualLogic;
	}

	public AbstractFormulaElement getElementByName(String elementName) {
		if (elementName == null) {
			return null;
		}

		Iterator<AbstractFormulaElement> it = getElementsIterator();
		while (it.hasNext()) {
			AbstractFormulaElement element = it.next();

			if (elementName.equals(element.getElementName())) {
				return element;
			}
		}
		return null;
	}

	public int getElementIndexByName(String elementName) {
		if (elementName == null) {
			return -1;
		}

		Iterator<AbstractFormulaElement> it = getElementsIterator();
		int i = 0;
		while (it.hasNext()) {
			AbstractFormulaElement element = it.next();

			if (elementName.equals(element.getElementName())) {
				return i;
			}
			i++;
		}
		return -1;
	}

	@Override
	public Iterator<AbstractFormulaElement> iterator() {
		List<AbstractFormulaElement> elements = getElements();
		return elements == null ? new ArrayList<AbstractFormulaElement>().iterator() : elements.iterator();
	}


	public Iterator<AbstractFormulaElement> getElementsIterator() {
		return iterator();
	}

	protected void convertElements() {
		List<BaseMap> elements = getListValue(FIELD_FE_ELEMENTS);
		if (elements != null) {
			setElements(elements
					.stream()
					.map(map -> createElement(map))
					.collect(Collectors.toList())
			);
		}
	}

	public void setElements(List<AbstractFormulaElement> elements) {
		setListValue(FIELD_FE_ELEMENTS, elements);
	}

	public void addElement(AbstractFormulaElement element) {
		getElements().add(element);
	}


	public abstract AbstractFormulaElement createElement();

	public abstract AbstractFormulaElement createElement(BaseMap element);

	public abstract boolean isPricingLogic();

	public abstract boolean isWorkflowLogic();

	public boolean isHeaderLogic() {
		IFormulaNature nature = getFormulaNatureObject();

		return QUOTE_HEADER.equals(nature)
				|| CONTRACT_HEADER.equals(nature)
				|| REBATE_AGREEMENT_HEADER.equals(nature)
				|| CO_HEADER.equals(nature)
				|| COMPENSATION_HEADER.equals(nature)
				|| CFO_HEADER.equals(nature);
	}


	public String getUniqueName() {
		return getStringValue(FIELD_UNIQUENAME);
	}

	public void setUniqueName(String uniqueName) {
		setStringValue(FIELD_UNIQUENAME, uniqueName);
	}

	public String getLabel() {
		return getStringValue(FIELD_LABEL);
	}

	public void setLabel(String label) {
		setStringValue(FIELD_LABEL, label);
	}

	public List<AbstractFormulaElement> getElements() {
		return getListValue(FIELD_FE_ELEMENTS);
	}

	public String getValidAfter() {
		return getStringValue(FIELD_VALIDAFTER);
	}

	public void setValidAfter(String validAfter) {
		setStringValue(FIELD_VALIDAFTER, validAfter);
	}

	public String getStatus() {
		return getStringValue(FIELD_STATUS);
	}

	public void setStatus(String status) {
		setStringValue(FIELD_STATUS, status);
	}

	public Long getVersion() {
		return getLongValue(FIELD_VERSION);
	}

	public void setVersion(Long version) {
		setLongValue(FIELD_VERSION, version);
	}

	public String getTypedId() {
		return getStringValue(FIELD_TYPEDID);
	}

	public void setTypedId(String typedId) {
		setStringValue(FIELD_TYPEDID, typedId);
	}

	public String getSimulationSet() {
		return getStringValue(FIELD_SIMULATIONSET);
	}

	public void setSimulationSet(String simulationSet) {
		setStringValue(FIELD_SIMULATIONSET, simulationSet);
	}

	public String getFormulaType() {
		return getStringValue("formulaType");
	}

	public void setFormulaType(String formulaType) {
		setStringValue("formulaType", formulaType);
	}

	public String getFormulaNature() {
		return getStringValue(FIELD_FE_FORMULANATURE);
	}

	public String getUserGroupEdit(){
		return getStringValue(PriceFxInterface.FIELD_USERGROUPEDIT);
	}
	public void setUserGroupEdit(String userGroupEdit){
		setStringValue(PriceFxInterface.FIELD_USERGROUPEDIT, userGroupEdit);
	}

	public String getUserGroupViewDetails(){
		return getStringValue(PriceFxInterface.FIELD_USERGROUPVIEWDETAILS);
	}
	public void setUserGroupViewDetails(String userGroupViewDetails){
		setStringValue(PriceFxInterface.FIELD_USERGROUPVIEWDETAILS, userGroupViewDetails);
	}

	public void setFormulaNature(String formulaNature) {
		setStringValue(FIELD_FE_FORMULANATURE, formulaNature);
	}

	@JsonIgnore
	public abstract IFormulaNature getFormulaNatureObject();
}
